<?php

namespace Drupal\social_auth_entra_id\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;
use GuzzleHttp\ClientInterface;
use Drupal\user\Entity\User;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Language\LanguageManagerInterface;

class SocialAuthEntraIdController extends ControllerBase {

  protected $configFactory;
  protected $httpClient;
  protected $messenger;

  public function __construct(ConfigFactoryInterface $config_factory, ClientInterface $http_client, MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
    $this->messenger = $messenger;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('http_client'),
      $container->get('messenger')
    );
  }

  /**
   * Redirects the user to the Microsoft login page.
   */
  public function redirectToMicrosoft() {
    $config = $this->configFactory->get('social_auth_entra_id.settings');
    $client_id = $config->get('client_id');
    $tenant_id = $config->get('tenant_id');

    if (empty($client_id) || empty($tenant_id)) {
      // Add an alert message for empty configuration.
      $this->messenger->addError($this->t('Empty configuration. Please contact the site administrator.'));

      // Redirect to the user login page with the current language.
      $current_language = \Drupal::languageManager()->getCurrentLanguage()->getId();
      return new RedirectResponse(Url::fromRoute('user.login', [], ['language' => $current_language])->toString());
    }

    // Use Url::fromRoute() to generate the redirect URI
    $redirect_uri = Url::fromRoute('social_auth_entra_id.callback', [], ['absolute' => TRUE])->toString();
    $scopes = 'openid profile email';

    // Construct the URL for the Microsoft login.
    $url = "https://login.microsoftonline.com/$tenant_id/oauth2/v2.0/authorize?" . http_build_query([
      'client_id' => $client_id,
      'response_type' => 'code',
      'redirect_uri' => $redirect_uri,
      'scope' => $scopes,
    ]);

    // Redirect the user to Microsoft login using TrustedRedirectResponse.
    return new TrustedRedirectResponse($url);
  }

  /**
   * Handles the callback from Microsoft Entra ID after user authorization.
   */
  public function handleMicrosoftCallback(Request $request) {
    $code = $request->query->get('code');

    if ($code) {
      // Retrieve settings from configuration.
      $config = $this->configFactory->get('social_auth_entra_id.settings');
      $client_id = $config->get('client_id');
      $client_secret = $config->get('client_secret');
      $tenant_id = $config->get('tenant_id');
      $redirect_uri = Url::fromRoute('social_auth_entra_id.callback', [], ['absolute' => TRUE])->toString();
      $login_behavior = $config->get('login_behavior');
      $allowed_domains = array_map('trim', explode(',', $config->get('allowed_domains') ?? ''));

      try {
        // Exchange the code for an access token.
        $response = $this->httpClient->post("https://login.microsoftonline.com/$tenant_id/oauth2/v2.0/token", [
          'form_params' => [
            'client_id' => $client_id,
            'client_secret' => $client_secret,
            'code' => $code,
            'redirect_uri' => $redirect_uri,
            'grant_type' => 'authorization_code'
          ],
        ]);

        $data = json_decode($response->getBody()->getContents(), TRUE);

        // Check if access_token exists in response
        if (isset($data['access_token'])) {
          $access_token = $data['access_token'];
        } else {
          throw new \Exception('Access token not received from Microsoft.');
        }

        // Retrieve user info from Microsoft Graph.
        $profile_response = $this->httpClient->get('https://graph.microsoft.com/v1.0/me', [
          'headers' => ['Authorization' => 'Bearer ' . $access_token]
        ]);

        $profile_data = json_decode($profile_response->getBody()->getContents(), TRUE);

        // Check if the user's email exists in the profile data
        if (isset($profile_data['mail'])) {
          $user_email = $profile_data['mail'];
          $user_email_domain = substr(strrchr($user_email, "@"), 1);

          // Check if the email domain is allowed.
          if (!empty($allowed_domains) && !in_array($user_email_domain, $allowed_domains)) {
            $this->messenger->addError($this->t('Your email domain is not allowed to register or log in.'));
            return new RedirectResponse(Url::fromRoute('<front>')->toString());
          }

          // Check if the user exists in Drupal.
          $existing_user = user_load_by_mail($user_email);

          if (!$existing_user) {
            if ($login_behavior == 'register_and_login') {
              // Register a new user and log them in.
              $new_user = User::create([
                'name' => $profile_data['displayName'] ?? $user_email, // Fallback to email if displayName is not set
                'mail' => $user_email,
                'status' => 1,
              ]);
              $new_user->save();
              user_login_finalize($new_user);
              $this->messenger->addStatus($this->t('Account created and logged in successfully.'));
            } else {
              $this->messenger->addError($this->t('Login failed. The account does not exist. Please sign up to create a new account.'));
              return new RedirectResponse(Url::fromRoute('<front>')->toString());
            }
          } else {
            // Log in the existing user.
            user_login_finalize($existing_user);
            $this->messenger->addStatus($this->t('Logged in successfully.'));
          }
        } else {
          throw new \Exception('User email not found in profile data.');
        }
      } catch (\Exception $e) {
        // Log the exception message and details
        $this->messenger->addError($this->t('An error occurred during login. Please try again.'));
        \Drupal::logger('social_auth_entra_id')->error('Error during Microsoft login: @message', ['@message' => $e->getMessage()]);
      }
    } else {
      $this->messenger->addError($this->t('Authorization code is missing. Unable to log in.'));
    }

    // Redirect to the front page after handling callback or errors.
    return new RedirectResponse(Url::fromRoute('<front>')->toString());
  }

}
